﻿using Phidget22;
using Phidget22.Events;
using Phidget22.ExampleUtils;
using System;
using System.Linq;
using System.Windows.Forms;

namespace ResistanceInput_Example {
	public partial class Form1 : Form {
		CommandLineOpen openArgs;
		ResistanceInput openCh = null;
		formCleaner cleaner;
		ResistanceInput res = null;
		private ErrorEventBox errorBox;
		private DataGraphBox resistanceGraphBox;

		public Form1() {
			InitializeComponent();
			openArgs = new CommandLineOpen(this);
			cleaner = new formCleaner(this);
		}

		public Form1(ResistanceInput ch) {
			InitializeComponent();
			openCh = ch;
			cleaner = new formCleaner(this);
		}

		private void Form1_Load(object sender, EventArgs e) {
			errorBox = new ErrorEventBox(this);
			resistanceGraphBox = new DataGraphBox(this, "Resistance");

			this.Size = this.MinimumSize;

			res = openCh ?? openArgs.makeChannel<ResistanceInput>();

			//setup event handlers for the phidget
			res.Attach += res_attach;
			res.Detach += res_detach;
			res.Error += res_error;
			res.ResistanceChange += res_change;
			res.PropertyChange += res_propertychange;

			try {
				res.Open();
			} catch (PhidgetException ex) { errorBox.addMessage("Error opening device: " + ex.Message); }
		}

		private void Form1_FormClosing(object sender, FormClosingEventArgs e) {

			res.Attach -= res_attach;
			res.Detach -= res_detach;
			res.Error -= res_error;
			res.ResistanceChange -= res_change;
			res.PropertyChange -= res_propertychange;
			res.Close();
		}

		void res_attach(object sender, Phidget22.Events.AttachEventArgs e) {
			phidgetInfoBox1.FillPhidgetInfo((Phidget)sender);

			ResistanceInput attachedDevice = (ResistanceInput)sender;

			resistanceGraphBox.SetTitle(DataGraphBox.GenerateTitle("Resistance", (Phidget)attachedDevice));

			dataIntervalTrk.Unit = " ms";
			dataIntervalTrk.initializeRange(attachedDevice.MinDataInterval, attachedDevice.MaxDataInterval);
			dataIntervalTrk.Value = attachedDevice.DataInterval;

			//differing max/min means the interval is a settable configuration
			if (attachedDevice.MaxDataInterval == attachedDevice.MinDataInterval)
				dataIntervalTrk.Enabled = false;

			changeTriggerTrk.trkText = changeTriggerTrk.minTxt = attachedDevice.MinResistanceChangeTrigger.ToString() + " Ω";
			changeTriggerTrk.Minimum = attachedDevice.MinResistanceChangeTrigger;
			changeTriggerTrk.Maximum = attachedDevice.MaxResistanceChangeTrigger;
			changeTriggerTrk.maxTxt = attachedDevice.MaxResistanceChangeTrigger.ToString() + " Ω";
			changeTriggerTrk.Value = 0;
			changeTriggerTrk.trkText = attachedDevice.ResistanceChangeTrigger.ToString("F0") + " Ω";

#if DEBUG
			if (attachedDevice.DeviceID != DeviceID.TP_DIAGNOSER) {
#endif
			additionalSettings.Visible = true;
			rtdWireSetupPanel.Visible = true;
			rtdWireSetupCombo.SelectedIndexChanged -= rtdWireSetupCombo_SelectedIndexChanged;
			rtdWireSetupCombo.DataSource = typeof(RTDWireSetup).ToList();
			rtdWireSetupCombo.SelectedValue = attachedDevice.RTDWireSetup;
			rtdWireSetupCombo.SelectedIndexChanged += rtdWireSetupCombo_SelectedIndexChanged;
#if DEBUG
			}
#endif

			settingsBox.Visible = true;
			outputBox.Visible = true;
		}


		void res_detach(object sender, Phidget22.Events.DetachEventArgs e) {
			phidgetInfoBox1.Clear();
			cleaner.clean();

			settingsBox.Visible = false;
			outputBox.Visible = false;
			rtdWireSetupPanel.Visible = false;
			additionalSettings.Visible = false;
			dataIntervalTrk.TrueValue = -1;
		}

		void res_error(object sender, Phidget22.Events.ErrorEventArgs e) {
			errorBox.addMessage(e.Description);

			if (e.Code == ErrorEventCode.Saturation)
				resistanceGraphBox.addGap();
		}

		private void res_propertychange(object sender, PropertyChangeEventArgs e) {

			switch (e.PropertyName) {
			case "DataInterval":
				if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL) {
					dataIntervalTrk.Value = res.DataInterval;
					updateGraphDataInterval();
				}
				break;
			case "ResistanceChangeTrigger":
				changeTriggerTrk.Value = res.ResistanceChangeTrigger;
				break;
			case "DataRate":
				if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE) {
					dataIntervalTrk.Value = res.DataRate;
					updateGraphDataInterval();
				}
				break;
			case "RTDWireSetup":
				rtdWireSetupCombo.SelectedValue = res.RTDWireSetup;
				break;
			}
		}
		void res_change(object sender, Phidget22.Events.ResistanceInputResistanceChangeEventArgs e) {
			resistanceTxt.Text = e.Resistance.ToString() + " Ω";
			resistanceGraphBox.addData(e.Resistance, res.DataInterval);
		}

		private void rtdWireSetupCombo_SelectedIndexChanged(object sender, EventArgs e) {
			try {
				res.RTDWireSetup = (RTDWireSetup)Enum.Parse(typeof(RTDWireSetup), rtdWireSetupCombo.SelectedValue.ToString());
			} catch (PhidgetException ex) { errorBox.addMessage("Error setting RTD type: " + ex.Message); }
		}
		private void updateGraphDataInterval() {
			try {
				if (res.ResistanceChangeTrigger == 0)
					resistanceGraphBox.DataInterval = (1000.0 / res.DataRate);
				else
					resistanceGraphBox.DataInterval = null;
			} catch (PhidgetException ex) { errorBox.addMessage("Error updating data interval: " + ex.Message); }
		}
		private void resistanceGraphButton_Click(object sender, EventArgs e) {
			if (!resistanceGraphBox.Visible)
				resistanceGraphBox.Show();
		}
		private void dataIntervalTrk_KeyOverride(object sender, KeyEventArgs e) {
			e.SuppressKeyPress = true;
			dataIntervalTrk.keyOverride(sender, e);
		}

		private void slider_ValueChangeEvent(object sender, EventArgs e) {
			if (((PhidgetsControlLibrary.CustomScroll)sender).isFixedRange) //datainterval control
			{
				if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL) {
					try {
						double temp = dataIntervalTrk.Value;
						res.DataInterval = (int)temp;
						if (res.DataInterval < (int)temp) {
							while (res.DataInterval != (int)temp) {
								temp++;
								res.DataInterval = (int)temp;
							}
						} else {
							while (res.DataInterval != (int)temp) {
								temp--;
								res.DataInterval = (int)temp;
							}
						}

						dataIntervalTrk.TrueValue = res.DataInterval;
						dataIntervalTrk.Value = res.DataInterval;



						updateGraphDataInterval();
					} catch (Exception ex) { errorBox.addMessage("Error setting data interval: " + ex.Message); }
				} else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE) {
					try {
						res.DataRate = dataIntervalTrk.Value;
						updateGraphDataInterval();
					} catch (Exception ex) { errorBox.addMessage("Error setting data rate: " + ex.Message); }
				}
			} else //changetrigger control
			  {
				try {
					res.ResistanceChangeTrigger = changeTriggerTrk.Value;
					updateGraphDataInterval();
				} catch (Exception ex) { errorBox.addMessage("Error setting change trigger: " + ex.Message); }
			}
		}
		private void dataIntervalTrk_ModeSwitch(object sender, EventArgs e) {
			dataIntervalTrk.SwitchModes();
			if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.RATE) {
				try {
					dataIntervalTrk.isIntValue = false;
					dataIntervalTrk.Unit = "Hz";
					dataIntervalTrk.trkText = res.DataRate.ToString() + "Hz";
					dataIntervalTrk.Value = res.DataRate;
				} catch (Exception ex) { errorBox.addMessage("Error setting data rate: " + ex.Message); }

			} else if (dataIntervalTrk.TimingMode == PhidgetsControlLibrary.CustomScroll.Modes.INTERVAL) {
				try {
					dataIntervalTrk.isIntValue = true;
					if (res.DeviceID != DeviceID.PN_1046)
						dataIntervalTrk.Unit = "ms";
					dataIntervalTrk.trkText = res.DataInterval.ToString() + "ms";
					dataIntervalTrk.Value = res.DataInterval;
				} catch (Exception ex) { errorBox.addMessage("Error setting data interval: " + ex.Message); }
			}
			dataIntervalTrk.TrueValue = dataIntervalTrk.Value;
		}
	}
}